@@ -1,40 +1,40 @@
 # BEGIN BPS TAGGED BLOCK {{{
-# 
+#
 # COPYRIGHT:
-# 
-# This software is Copyright (c) 1996-2009 Best Practical Solutions, LLC
-#                                          <jesse@bestpractical.com>
-# 
+#
+# This software is Copyright (c) 1996-2011 Best Practical Solutions, LLC
+#                                          <sales@bestpractical.com>
+#
 # (Except where explicitly superseded by other copyright notices)
-# 
-# 
+#
+#
 # LICENSE:
-# 
+#
 # This work is made available to you under the terms of Version 2 of
 # the GNU General Public License. A copy of that license should have
 # been provided with this software, but in any event can be snarfed
 # from www.gnu.org.
-# 
+#
 # This work is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301 or visit their web page on the internet at
 # http://www.gnu.org/licenses/old-licenses/gpl-2.0.html.
-# 
-# 
+#
+#
 # CONTRIBUTION SUBMISSION POLICY:
-# 
+#
 # (The following paragraph is not intended to limit the rights granted
 # to you to modify and distribute this software under the terms of
 # the GNU General Public License and is only of importance to you if
 # you choose to contribute your changes and enhancements to the
 # community by submitting them to Best Practical Solutions, LLC.)
-# 
+#
 # By intentionally submitting any modifications, corrections or
 # derivatives to this work, or any other work intended for use with
 # Request Tracker, to Best Practical Solutions, LLC, you confirm that
@@ -43,7 +43,7 @@
 # royalty-free, perpetual, license to use, copy, create derivative
 # works based on those contributions, and sublicense and distribute
 # those contributions and any derivatives thereof.
-# 
+#
 # END BPS TAGGED BLOCK }}}
 
 use strict;
@@ -124,7 +124,6 @@ sub CleanEnv {
 
     my $CurrentUser; # shared betwen GetCurrentUser and loc
 
-# {{{ sub GetCurrentUser 
 
 =head2 GetCurrentUser
 
@@ -144,7 +143,7 @@ sub GetCurrentUser  {
     #If the current user is 0, then RT will assume that the User object
     #is that of the currentuser.
 
-    $CurrentUser = new RT::CurrentUser();
+    $CurrentUser = RT::CurrentUser->new();
     $CurrentUser->LoadByGecos($Gecos);
     
     unless ($CurrentUser->Id) {
@@ -153,10 +152,8 @@ sub GetCurrentUser  {
 
     return($CurrentUser);
 }
-# }}}
 
 
-# {{{ sub loc 
 
 =head2 loc
 
@@ -168,12 +165,10 @@ sub loc {
     die "No current user yet" unless $CurrentUser ||= RT::CurrentUser->new;
     return $CurrentUser->loc(@_);
 }
-# }}}
 
 }
 
 
-# {{{ sub GetMessageContent
 
 =head2 GetMessageContent
 
@@ -200,7 +195,7 @@ sub GetMessageContent {
     
     #Load the sourcefile, if it's been handed to us
     if ($source) {
-	open (SOURCE, "<$source");
+	open( SOURCE, '<', $source ) or die $!;
 	@lines = (<SOURCE>);
 	close (SOURCE);
     }
@@ -226,7 +221,7 @@ sub GetMessageContent {
 	system ($ENV{'EDITOR'}, $filename);
     }	
     
-    open (READ, "<$filename");
+    open( READ, '<', $filename ) or die $!;
     my @newlines = (<READ>);
     close (READ);
 
@@ -235,9 +230,7 @@ sub GetMessageContent {
     
 }
 
-# }}}
 
-# {{{ sub debug
 
 sub debug {
     my $val = shift;
@@ -253,12 +246,21 @@ sub debug {
     }	
 }
 
-# }}}
+sub ShowHelp {
+    my $self = shift;
+    my %args = @_;
+    require Pod::Usage;
+    Pod::Usage::pod2usage(
+        -message => $args{'Message'},
+        -exitval => $args{'ExitValue'} || 0, 
+        -verbose => 99,
+        -sections => $args{'Sections'} || ($args{'ExitValue'}
+            ? 'NAME|USAGE'
+            : 'NAME|USAGE|OPTIONS|DESCRIPTION'
+        ),
+    );
+}
 
-
-eval "require RT::Interface::CLI_Vendor";
-die $@ if ($@ && $@ !~ qr{^Can't locate RT/Interface/CLI_Vendor.pm});
-eval "require RT::Interface::CLI_Local";
-die $@ if ($@ && $@ !~ qr{^Can't locate RT/Interface/CLI_Local.pm});
+RT::Base->_ImportOverlays();
 
 1;
